/*
AutoConvert, a Chinese HZ/GB/Big5 encodings auto-converter
Copyright (C) 1999  ڹ  Yu Guanghui <ygh@debian.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
*/
/* 
* author: Yu Guanghui <ygh@debian.org>
*	  Network Center
* 	  Dalian Univ. of Tech.	
*	  1999.5
*/

/*
 *Author: Ha Shao <hashao@china.com>
 *Date:	2000.08.30
 */

/* Judge Encoding based on character frequency.
 * We will use 2 hash tables generated from GNU gperf
 */

// Platform settings
#include <OggOs.h>
#include <OggShared.h>

//#include "zhstatis.h"

/////////////////////////////////////////////////
// Locals
//

// not used, #define GBTOPPER        3.500849        /* Top GB frequency. */
// not used, #define B5TOPPER        3.803567        /* Top Big5 frequency. */
// not used, #define MAX_MODULE      16
// not used, #define MAX_BUFFER      8192

struct charHz { char *name; double percent; };

/////////////////////////////////////////////////
// Local Function Prototypes
//

static unsigned int         b5Hash ( register const char *str, register unsigned int len );
static unsigned int         gbHash ( register const char *str, register unsigned int len );
static const struct charHz* inBig5( register const char *str, register unsigned int len );
static const struct charHz* inGB( register const char *str, register unsigned int len );
static int                  j_code( const char* buff, int count );
static int                  j_code3( const char * buff, int count );

/* ANSI-C code produced by gperf version 2.7.1 (19981006 egcs) */
/* Command-line: gperf -L ANSI-C -I -t -H gbHash -N inGB -C -E -o -n -c -D gbpercent.txt  */
/****************************************************
 400 Most frequently used GB chars. 
 ****************************************************/
/* maximum key range = 991, duplicates = 5 */

/////////////////////////////////////////////////
// Global Functions
//

EXPORT_C int jcode(const char* buff)
{
	return j_code(buff, strlen(buff));
}

static int j_code(const char* buff, int count)
{
	const unsigned char * phz;
	int c_gb=0;
	int c_big5=0;

	char HZ_START[]="~{";
	char HZ_END[]="~}";


/* first we look up ""  and "" ,both gb and big5
 * in the text.
 */
	for(phz=(unsigned char * /*FIXIT*/)buff;phz<((unsigned char * /*FIXIT*/)buff+count);phz++){
		if(*phz & 0x80){
			if((*phz==0xB5&&*(phz+1)==0xC4) || ((*phz==0xCE)&&*(phz+1)==0xD2)){
				c_gb++;
				phz++;
				continue;
			}else if((*phz==0xAA&&*(phz+1)==0xBA)|| ((*phz==0xA7)&&*(phz+1)==0xDA)){
				c_big5++;
				phz++;
				continue;
			}
			phz++;
		}
	}

	if(c_gb > c_big5){
		return GB_CODE;
	}else if (c_gb ==  c_big5){ //c_gb == 0,c_big5==0
		if(strstr(buff,HZ_START)!=NULL && strstr(buff,HZ_END)!=NULL){
			return HZ_CODE;
		}
		/*There is not "" and "" in the text
		 *So we test the text with a 400 words table.
		 */
		return j_code3(buff,count);		
	}else{
		return BIG5_CODE;
	}
}

/////////////////////////////////////////////////
// Local Functions
//

#ifdef __GNUC__
__inline
#endif
static unsigned int
gbHash (register const char *str, register unsigned int len)
{
  static const unsigned short asso_values[] =
    {
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991, 991, 991, 991, 991, 991, 991, 991, 991, 991,
      991,  91, 425, 320, 148,  70, 510,  85, 203, 160,
        6,  18, 480, 223, 113, 345,  28, 120, 320, 390,
       75,  20, 275, 380, 230,  50, 433,  15, 145,   5,
      455, 460, 163, 180, 338,  40,   8, 508, 510,  45,
      325,  85,  30, 140, 105, 305,   3, 225, 100, 403,
      113, 480,  25, 205,   0, 250, 393, 371,  63, 260,
      268, 991,  10, 470, 405, 268, 348,  76, 285, 131,
      408, 398, 991, 125, 330, 126, 388, 503,  51, 148,
      350,  46,   1, 166,  15,  16, 485, 106, 458,  41,
      103, 221,  21,  61, 500, 991
    };
  return asso_values[(unsigned char)str[len - 1]] + asso_values[(unsigned char)str[0]];
}

#ifdef __GNUC__
__inline
#endif
static const struct charHz *
inGB (register const char *str, register unsigned int len)
{
  enum
    {
      TOTAL_KEYWORDS = 400,
      MIN_WORD_LENGTH = 2,
      MAX_WORD_LENGTH = 2,
      MIN_HASH_VALUE = 0,
      MAX_HASH_VALUE = 990
    };

  static const struct charHz wordlist[] =
    {
      {"",	0.151748},
      {"",	0.062792},
      {"֪",	0.176508},
      {"Ϊ",	0.442767},
      {"",	0.214763},
      {"",	0.213959},
      {"ֻ",	0.229438},
      {"",	0.085280},
      {"λ",	0.128747},
      {"",	0.073778},
      {"",	0.094296},
      {"",	0.168043},
      {"",	0.514231},
      {"",	0.063487},
      {"",	3.500849},
      {"",	0.211674},
      {"Ԫ",	0.071841},
      {"",	0.140707},
      {"ʽ",	0.057680},
      {"",	0.167690},
      {"",	0.072121},
      {"",	0.146944},
      {"",	0.070839},
      {"",	0.081974},
      {"ʵ",	0.156769},
      {"",	0.083052},
      {"",	0.382771},
      {"û",	0.244614},
      {"",	0.057849},
      {"ë",	0.075090},
      {"",	0.396983},
      {"",	0.093383},
      {"",	0.058726},
      {"",	0.233198},
      {"",	0.092784},
      {"",	0.188655},
      {"",	0.054738},
      {"",	0.111181},
      {"ǰ",	0.200232},
      {"",	1.345411},
      {"",	0.129442},
      {"",	0.121678},
      {"δ",	0.054546},
      {"ʹ",	0.118288},
      {"",	0.123170},
      {"",	0.436374},
      {"",	0.495768},
      {"",	0.174189},
      {"",	0.964866},
      {"ɽ",	0.103285},
      {"",	0.091243},
      {"",	0.063780},
      {"",	0.152685},
      {"",	0.717652},
      {"",	0.058755},
      {"ɫ",	0.074190},
      {"",	0.089328},
      {"",	0.204736},
      {"",	0.273937},
      {"",	0.117537},
      {"",	0.117812},
      {"",	0.327524},
      {"֮",	0.327879},
      {"",	0.127394},
      {"ô",	0.276714},
      {"",	1.014949},
      {"",	0.137512},
      {"Ҫ",	0.370943},
      {"ֱ",	0.075193},
      {"",	0.064573},
      {"̫",	0.103034},
      {"",	0.095783},
      {"",	0.151640},
      {"һ",	1.703619},
      {"",	0.101242},
      {"",	0.091301},
      {"ҵ",	0.134539},
      {"",	0.103181},
      {"",	0.126292},
      {"",	0.060073},
      {"",	0.349432},
      {"",	0.222406},
      {"",	0.073591},
      {"ʮ",	0.211422},
      {"Ա",	0.097189},
      {"",	0.152480},
      {"",	0.123866},
      {"ʱ",	0.412130},
      {"",	0.078221},
      {"",	0.624527},
      {"",	0.061338},
      {"",	0.098381},
      {"Ǯ",	0.055097},
      {"˵",	0.508134},
      {"",	0.075883},
      {"",	0.055787},
      {"",	0.061514},
      {"",	0.115919},
      {"",	0.075698},
      {"",	0.143753},
      {"",	0.073937},
      {"",	0.083070},
      {"д",	0.056349},
      {"",	0.078525},
      {"",	0.104545},
      {"",	0.079151},
      {"",	0.074843},
      {"",	0.197660},
      {"",	0.421135},
      {"",	0.176988},
      {"",	0.064201},
      {"",	0.343557},
      {"",	0.077728},
      {"С",	0.293018},
      {"",	0.114808},
      {"",	0.063376},
      {"",	0.128778},
      {"",	0.149985},
      {"",	0.203868},
      {"",	0.305242},
      {"",	0.152936},
      {"",	0.074596},
      {"ս",	0.076361},
      {"",	0.059639},
      {"",	0.057653},
      {"",	0.158364},
      {"",	0.073128},
      {"",	0.065122},
      {"",	0.059687},
      {"",	0.070030},
      {"",	0.061845},
      {"",	0.169370},
      {"",	0.143764},
      {"",	0.058127},
      {"ָ",	0.068890},
      {"",	0.055467},
      {"",	0.074475},
      {"",	0.055267},
      {"",	0.100845},
      {"",	0.642527},
      {"",	0.113407},
      {"",	0.670268},
      {"",	0.072306},
      {"ϵ",	0.085791},
      {"ԭ",	0.089937},
      {"",	0.074014},
      {"",	0.087664},
      {"ˮ",	0.102945},
      {"չ",	0.063943},
      {"ת",	0.059482},
      {"",	0.324818},
      {"Щ",	0.166013},
      {"",	0.142495},
      {"",	0.143041},
      {"",	0.062711},
      {"",	0.063875},
      {"",	0.084231},
      {"",	0.070334},
      {"",	0.340950},
      {"",	0.253537},
      {"",	0.089789},
      {"",	0.066471},
      {"",	0.342850},
      {"˼",	0.068379},
      {"",	0.067758},
      {"",	0.061514},
      {"",	0.058356},
      {"",	0.121345},
      {"",	0.084066},
      {"",	0.084736},
      {"",	0.311761},
      {"Զ",	0.066691},
      {"",	0.134213},
      {"",	0.089264},
      {"ʶ",	0.056331},
      {"",	0.101912},
      {"̨",	0.078054},
      {"",	0.636581},
      {"",	0.300629},
      {"",	0.058334},
      {"",	0.076271},
      {"",	0.146378},
      {"",	0.067145},
      {"",	1.283668},
      {"",	0.084392},
      {"",	0.094190},
      {"",	0.060622},
      {"",	0.079391},
      {"",	0.088464},
      {"",	0.065369},
      {"",	1.307755},
      {"",	0.080068},
      {"",	0.402590},
      {"Ȼ",	0.251950},
      {"ס",	0.097438},
      {"ȫ",	0.151277},
      {"",	0.136630},
      {"",	0.064102},
      {"",	0.090869},
      {"ʲ",	0.130740},
      {"",	0.217789},
      {"",	0.081322},
      {"",	0.088088},
      {"",	0.224425},
      {"",	0.055379},
      {"",	0.107695},
      {"",	0.826706},
      {"",	0.152597},
      {"",	0.073580},
      {"",	0.196038},
      {"",	0.229908},
      {"",	0.077287},
      {"",	0.088749},
      {"",	0.135949},
      {"",	0.060397},
      {"",	0.168552},
      {"",	0.067138},
      {"",	0.084762},
      {"",	0.202303},
      {"",	0.185462},
      {"",	0.182608},
      {"",	0.053845},
      {"",	0.078045},
      {"",	0.126537},
      {"ȥ",	0.330934},
      {"",	0.117460},
      {"",	0.084711},
      {"",	0.072738},
      {"ȴ",	0.105046},
      {"",	0.056342},
      {"",	0.083728},
      {"",	0.064229},
      {"",	0.156121},
      {"",	0.089804},
      {"",	0.054791},
      {"",	0.092669},
      {"",	0.430809},
      {"",	0.072425},
      {"",	0.061534},
      {"",	0.125012},
      {"",	0.252867},
      {"",	0.103862},
      {"",	0.068245},
      {"",	0.064196},
      {"",	0.131700},
      {"",	0.158959},
      {"Ҳ",	0.412956},
      {"",	0.061082},
      {"",	0.060972},
      {"",	0.088065},
      {"",	0.131476},
      {"",	0.061942},
      {"",	0.057693},
      {"",	0.112545},
      {"",	0.169738},
      {"",	0.097747},
      {"",	0.188836},
      {"",	0.067301},
      {"",	0.103631},
      {"",	0.074605},
      {"",	0.087462},
      {"",	0.112757},
      {"Ժ",	0.064434},
      {"",	0.060794},
      {"",	0.161359},
      {"",	0.161229},
      {"",	1.081233},
      {"",	0.211932},
      {"Ŀ",	0.094628},
      {"",	0.100654},
      {"",	0.075649},
      {"",	0.253067},
      {"",	0.232004},
      {"",	0.104464},
      {"",	0.141283},
      {"",	0.202277},
      {"",	0.098824},
      {"",	0.071268},
      {"",	0.076121},
      {"ѧ",	0.313110},
      {"ʿ",	0.059531},
      {"",	0.262760},
      {"",	0.058312},
      {"",	0.075691},
      {"ÿ",	0.067385},
      {"",	0.059983},
      {"ǿ",	0.060747},
      {"ͨ",	0.095327},
      {"",	0.081732},
      {"",	0.060990},
      {"ƽ",	0.116858},
      {"",	0.183761},
      {"",	0.099417},
      {"",	0.208392},
      {"",	0.119434},
      {"",	0.055445},
      {"",	0.075475},
      {"Ȩ",	0.064165},
      {"",	0.238170},
      {"",	0.065223},
      {"",	0.216015},
      {"",	0.176671},
      {"ʦ",	0.098604},
      {"",	0.237050},
      {"",	0.310500},
      {"",	0.123004},
      {"",	0.103316},
      {"",	0.073529},
      {"",	0.748073},
      {"",	0.181841},
      {"",	0.069931},
      {"",	0.366675},
      {"",	0.106073},
      {"",	0.063793},
      {"",	0.055222},
      {"",	0.089414},
      {"",	0.197245},
      {"",	0.148374},
      {"",	0.846428},
      {"",	0.158459},
      {"",	0.099988},
      {"",	0.167521},
      {"",	0.077498},
      {"",	0.057761},
      {"˾",	0.084963},
      {"",	0.072820},
      {"",	0.297066},
      {"",	0.074446},
      {"",	0.075854},
      {"",	0.093914},
      {"",	0.165147},
      {"",	0.114573},
      {"Ц",	0.096279},
      {"",	0.116060},
      {"",	0.082472},
      {"",	0.178348},
      {"Ů",	0.141049},
      {"",	0.090741},
      {"",	0.109685},
      {"",	0.057889},
      {"",	0.122566},
      {"",	0.060534},
      {"",	0.058129},
      {"",	0.059348},
      {"",	0.090981},
      {"Ϣ",	0.056893},
      {"",	0.167708},
      {"",	0.085688},
      {"",	0.087272},
      {"",	0.067046},
      {"",	0.101187},
      {"",	0.058894},
      {"",	0.154598},
      {"",	0.145362},
      {"",	0.189995},
      {"",	0.270297},
      {"ͷ",	0.189444},
      {"",	0.136868},
      {"",	0.138951},
      {"ͳ",	0.074402},
      {"",	0.162739},
      {"",	0.134687},
      {"",	0.120122},
      {"",	0.089189},
      {"",	0.093070},
      {"",	0.594848},
      {"·",	0.089668},
      {"",	0.125810},
      {"",	0.201591},
      {"",	0.441396},
      {"",	0.260181},
      {"",	0.086397},
      {"",	0.440206},
      {"",	0.111459},
      {"",	0.119826},
      {"",	0.066006},
      {"",	0.335011},
      {"",	0.389616},
      {"",	0.155393},
      {"",	0.076789},
      {"",	0.143537},
      {"ͬ",	0.187525},
      {"",	0.152385},
      {"",	0.232956},
      {"",	0.063231},
      {"",	0.099303},
      {"",	0.093403},
      {"",	0.064661},
      {"",	0.235155},
      {"",	0.425756},
      {"",	0.075191},
      {"",	0.087173},
      {"Ʒ",	0.059628},
      {"Ӣ",	0.057878},
      {"",	0.062792},
      {"",	0.096854},
      {"",	0.114705},
      {"",	0.072145},
      {"",	0.061058},
      {"Ӧ",	0.089767}
    };

  static const short lookup[] =
    {
         0,   -1,   -1,    1,   -1,   -1,    2,   -1,
        -1,    3,   -1,    4,   -1,    5,   -1,    6,
         7,   -1,    8,   -1,    9,   10,   -1,   11,
        -1,   12,   13,   -1,   14,   -1,   15,   16,
        -1,   17,   -1,   18,   -1,   -1,   19,   -1,
        -1,   20,   -1,   21,   -1,   -1,   22,   -1,
        -1,   23,   24,   25,   -1,   26,   -1,   27,
        28,   -1,   29,   -1,   30,   31,   -1,   32,
        -1,   33,   34,   -1,   35,   -1,   36,   37,
        -1,   38,   -1,   39,   40,   -1, -482,   -1,
        43, -359,   -2,   44,   45,   46,   47,   -1,
        48,   -1,   49,   50,   -1,   51,   -1,   -1,
        52,   -1,   -1,   -1,   53,   54,   -1,   55,
        -1,   56,   57,   -1,   58,   59,   60,   61,
        -1,   62,   63,   64,   65,   -1,   66,   67,
        68,   69,   -1,   70,   -1,   71,   72,   -1,
        73,   -1,   74,   75,   -1,   76,   77,   78,
        79,   -1,   80,   -1,   81,   82,   -1,   83,
        -1,   84,   85,   -1,   86,   -1,   87,   88,
        -1,   89,   90,   91,   -1,   -1,   92,   -1,
        93,   94,   -1,   95,   -1,   96,   97,   -1,
        98,   -1,   99,  100,   -1,  101,   -1,  102,
       103,   -1,  104,   -1, -605,  107,   -1,  108,
        -1,  109,  110,   -1,  111,   -1,  112,  113,
        -1,  114,   -1,  115,  116,   -1,  117,   -1,
       118,  119,   -1,  120, -295,   -2,   -1,   -1,
       121,   -1,  122,  123,   -1,   -1,  124,  125,
       126,   -1,  127,   -1,  128,  129,   -1,  130,
        -1,  131,  132,   -1,  133,   -1,  134,  135,
        -1,  136,   -1,  137,  138,   -1,  139,   -1,
       140,  141,   -1,   -1,  142,  143,   -1,   -1,
       144,   -1,  145,  146,   -1,  147,   -1,  148,
       149,   -1,  150,   -1,  151,  152,   -1,   -1,
        -1,   -1,  153,   -1,  154,   -1,  155,  156,
        -1,  157,   -1,  158,  159,   -1,  160,   -1,
       161,   -1,   -1,  162,   -1,  163,  164,   -1,
       165,   -1,  166,   -1,   -1,  167,   -1,  168,
        -1,   -1,  169,   -1, -727,  172,   -1,  173,
        -1,  174,  175,   -1,  176,   -1,  177,  178,
        -1,  179,   -1,  180,  181,   -1,  182,   -1,
       183,  184,   -1,  185,   -1,  186, -230,   -2,
       187,   -1,  188,   -1,   -1,  189,   -1,  190,
       191,   -1,  192,   -1,  193,  194,   -1,  195,
        -1,  196,  197,   -1,  198,   -1,  199,  200,
        -1,  201,   -1,  202,  203,   -1,  204,   -1,
       205,  206,   -1,  207,   -1,  208,  209,   -1,
       210,   -1,  211,   -1,   -1,  212,   -1,  213,
        -1,   -1,  214,   -1,  215,  216,   -1,   -1,
       217,  218,  219,   -1,  220,   -1,  221,  222,
        -1,  223,   -1,  224,  225,   -1,  226,  227,
       228,  229,   -1,  230,   -1,  231,   -1,   -1,
       232,   -1, -822,  235,   -1,  236,   -1,  237,
       238,   -1,  239,   -1,  240, -167,   -2,  241,
        -1,  242,   -1,   -1,  243,   -1,  244,  245,
        -1,  246,  247,  248,  249,   -1,  250,   -1,
       251,  252,   -1,  253,  254,  255,   -1,   -1,
       256,   -1,  257,  258,   -1,  259,   -1,  260,
       261,   -1,  262,   -1,  263,  264,   -1,  265,
        -1,  266,  267,   -1,  268,   -1,  269,  270,
        -1,  271,   -1,  272,  273,   -1,  274,   -1,
       275,  276,   -1,  277,   -1,  278,   -1,   -1,
       279,   -1,  280,   -1,   -1,  281,   -1,  282,
        -1,   -1,  283,   -1,  284,   -1,   -1,  285,
        -1,  286,   -1,   -1,  287,   -1,  288,   -1,
        -1,  289,   -1,  290,  291,   -1,  292,   -1,
       293,   -1,   -1,  294,   -1,  295,  296,   -1,
       297,   -1,  298,   -1,   -1,  299,   -1,  300,
        -1,   -1,  301,   -1,  302,   -1,   -1,  303,
        -1,  304,   -1,   -1,  305,   -1, -952,  -94,
        -2,  308,   -1,  309,   -1,   -1,   -1,   -1,
       310,   -1,   -1,  311,   -1,  312,   -1,   -1,
       313,   -1,  314,   -1,   -1,  315,   -1,  316,
        -1,   -1,  317,   -1,  318,   -1,   -1,  319,
        -1,  320,   -1,   -1,  321,   -1,  322,   -1,
        -1,  323,   -1,  324,   -1,   -1,  325,  326,
       327,   -1,   -1,  328,   -1,  329,  330,   -1,
       331,   -1,  332,   -1,   -1,  333,   -1,  334,
        -1,   -1,   -1,   -1,  335,  336,   -1,  337,
        -1,  338,   -1,   -1,  339,   -1,  340,   -1,
        -1,   -1,   -1,  341,   -1,   -1,  342,   -1,
       343,   -1,   -1,   -1,   -1,  344,   -1,   -1,
        -1,   -1,  345,   -1,   -1,  346,   -1,  347,
        -1,   -1,  348,   -1,  349,   -1,   -1,  350,
        -1,  351,  352,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,  353,   -1,   -1,  354,   -1,
       355,   -1,   -1,   -1,   -1,  356,   -1,   -1,
       357,   -1,  358,   -1,   -1,   -1,   -1,  359,
        -1,   -1,   -1,   -1,  360,  361,   -1,   -1,
        -1,  362,   -1,   -1,   -1,   -1,  363,   -1,
        -1,  364,   -1,  365,   -1,   -1,  366,   -1,
       367,   -1,   -1,  368,   -1,   -1,   -1,   -1,
        -1,   -1,  369,   -1,   -1,  370,   -1,  371,
        -1,   -1,  372,   -1,   -1,   -1,   -1,   -1,
        -1,  373,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,  374,   -1,  375,   -1,   -1,  376,   -1,
       377,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,  378,
        -1,   -1,  379,   -1,  380,   -1,   -1,   -1,
        -1,  381,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,  382,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,  383,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,  384,   -1,   -1,
        -1,   -1,   -1,   -1,  385,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,  386,   -1,   -1,   -1,   -1,   -1,   -1,
       387,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,  388,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,  389,   -1,   -1,   -1,   -1,
       390,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
       391,   -1,  392,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,  393,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,  394,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,  395,   -1,   -1,   -1,   -1,
       396,   -1,  397,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,  398,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,  399
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      register int key = gbHash (str, len);

      if (key <= MAX_HASH_VALUE && key >= 0)
        {
          register int index = lookup[key];

          if (index >= 0)
            {
              register const char *s = wordlist[index].name;

              if (*str == *s && !strncmp (str + 1, s + 1, len - 1))
                return &wordlist[index];
            }
          else if (index < -TOTAL_KEYWORDS)
            {
              register int offset = - 1 - TOTAL_KEYWORDS - index;
              register const struct charHz *wordptr = &wordlist[TOTAL_KEYWORDS + lookup[offset]];
              register const struct charHz *wordendptr = wordptr + -lookup[offset + 1];

              while (wordptr < wordendptr)
                {
                  register const char *s = wordptr->name;

                  if (*str == *s && !strncmp (str + 1, s + 1, len - 1))
                    return wordptr;
                  wordptr++;
                }
            }
        }
    }
  return 0;
}

/* ANSI-C code produced by gperf version 2.7.1 (19981006 egcs) */
/* Command-line: gperf -L ANSI-C -I -t -H b5Hash -N inBig5 -C -E -o -n -c -D b5percent.txt  */
/****************************************************
 400 Most frequently used BIG5 chars in GB code space.
 The percentage is taken over all BIG5 code space.
****************************************************/

/* maximum key range = 986, duplicates = 4 */

#ifdef __GNUC__
__inline
#endif
static unsigned int
b5Hash (register const char *str, register unsigned int len)
{
  static const unsigned short asso_values[] =
    {
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986, 986, 986, 986, 986, 986, 986, 986, 986, 986,
      986,  81,   0, 166,   0,  55,  35,   5, 396,  21,
      116,  80, 265,  19, 986, 311,  28,  25, 345, 105,
      215,  95,  10, 415,  30, 468,  40, 110, 330,  80,
      155, 480, 180, 333,  21, 498, 188, 190,  89, 453,
      200, 228, 380,  13, 436, 475,  11, 278, 273, 430,
      280, 340, 120, 236, 183, 296, 488, 500, 456, 356,
      246, 120,  64, 491,  78, 285, 163, 461, 418, 288,
        4, 510, 326, 438, 383, 478,  76, 135, 330, 260,
      495,  45,  33,   1,  73, 501, 380, 433, 348, 505,
      425, 363,  23, 206, 195, 986
    };
  return asso_values[(unsigned char)str[len - 1]] + asso_values[(unsigned char)str[0]];
}

#ifdef __GNUC__
__inline
#endif
static const struct charHz *
inBig5 (register const char *str, register unsigned int len)
{
  enum
    {
      TOTAL_KEYWORDS = 400,
      MIN_WORD_LENGTH = 2,
      MAX_WORD_LENGTH = 2,
      MIN_HASH_VALUE = 0,
      MAX_HASH_VALUE = 985
    };

  static const struct charHz wordlist[] =
    {
      {"",	0.769287},
      {"",	0.306841},
      {"",	0.019016},
      {"",	0.088828},
      {"",	0.065464},
      {"",	0.031305},
      {"",	0.112574},
      {"",	0.028211},
      {"",	0.046708},
      {"",	0.015041},
      {"",	0.071498},
      {"",	0.025611},
      {"",	0.035635},
      {"",	0.016753},
      {"§",	0.018038},
      {"",	0.160736},
      {"",	0.090439},
      {"",	0.058479},
      {"",	0.037804},
      {"",	0.015738},
      {"",	0.014759},
      {"",	0.229607},
      {"",	0.034256},
      {"",	0.166304},
      {"",	0.093519},
      {"",	0.018394},
      {"",	0.045773},
      {"",	0.147260},
      {"",	0.017462},
      {"",	0.055894},
      {"",	0.015586},
      {"",	0.018907},
      {"",	0.019326},
      {"",	0.015277},
      {"",	0.039054},
      {"",	0.049332},
      {"",	0.057086},
      {"",	0.043716},
      {"",	0.423518},
      {"",	0.038273},
      {"",	0.016983},
      {"",	0.065455},
      {"",	0.043318},
      {"",	0.137466},
      {"",	0.404486},
      {"",	0.027452},
      {"",	0.031913},
      {"",	0.022180},
      {"",	0.108684},
      {"",	0.064182},
      {"",	0.066723},
      {"",	0.023091},
      {"",	0.113864},
      {"",	0.021874},
      {"",	0.015441},
      {"",	0.037946},
      {"",	0.028469},
      {"",	0.014519},
      {"",	0.069696},
      {"",	0.093334},
      {"",	0.128990},
      {"",	0.070255},
      {"",	0.083516},
      {"",	0.072818},
      {"",	0.016109},
      {"½",	0.018793},
      {"",	0.135395},
      {"",	0.098976},
      {"",	0.022989},
      {"",	0.185013},
      {"",	0.067999},
      {"",	0.041783},
      {"",	0.169524},
      {"",	0.098767},
      {"",	0.035493},
      {"",	0.031729},
      {"",	0.117938},
      {"",	0.019888},
      {"",	0.079914},
      {"",	0.041485},
      {"",	0.018574},
      {"",	0.016470},
      {"",	0.068123},
      {"",	0.014632},
      {"",	0.019986},
      {"",	0.061454},
      {"",	0.060330},
      {"",	0.016134},
      {"",	1.331823},
      {"",	0.311471},
      {"",	0.014513},
      {"",	0.033989},
      {"",	0.103263},
      {"",	0.127011},
      {"",	0.029026},
      {"",	0.015288},
      {"",	0.025526},
      {"",	0.137685},
      {"",	0.205378},
      {"",	1.647294},
      {"",	0.114942},
      {"",	0.049163},
      {"",	0.023332},
      {"",	0.029635},
      {"",	0.022240},
      {"¾",	0.030851},
      {"",	0.141643},
      {"",	0.120274},
      {"",	0.028860},
      {"",	0.102537},
      {"",	0.092950},
      {"",	0.051303},
      {"",	0.371579},
      {"",	0.014627},
      {"",	0.046636},
      {"",	0.049927},
      {"",	0.063111},
      {"",	0.019307},
      {"",	0.019892},
      {"",	0.082802},
      {"",	0.037599},
      {"",	0.069500},
      {"",	0.046553},
      {"",	0.022982},
      {"",	0.039860},
      {"ı",	0.125828},
      {"",	0.027650},
      {"",	0.014572},
      {"",	0.035152},
      {"",	0.028902},
      {"",	0.030283},
      {"",	0.018765},
      {"",	0.037953},
      {"",	0.015175},
      {"",	0.052447},
      {"",	0.016471},
      {"",	0.018525},
      {"",	0.016991},
      {"",	0.016309},
      {"",	0.023295},
      {"",	0.210737},
      {"ť",	0.109895},
      {"",	0.019304},
      {"",	0.016478},
      {"",	0.120442},
      {"",	0.029056},
      {"",	0.017140},
      {"",	0.205663},
      {"",	0.021996},
      {"",	0.026937},
      {"",	0.100132},
      {"",	0.076828},
      {"",	0.074701},
      {"",	0.070771},
      {"",	0.033967},
      {"",	0.135818},
      {"",	0.286923},
      {"",	0.021442},
      {"",	0.066187},
      {"",	0.154665},
      {"",	0.066513},
      {"",	0.019366},
      {"",	0.092406},
      {"ĵ",	0.019427},
      {"",	0.143276},
      {"ԣ",	0.019852},
      {"",	0.378802},
      {"",	0.029765},
      {"",	0.060734},
      {"ĳ",	0.060089},
      {"",	0.047633},
      {"",	0.017041},
      {"",	0.142548},
      {"",	0.041400},
      {"",	0.036977},
      {"",	0.030253},
      {"",	0.014722},
      {"Ū",	0.041732},
      {"",	0.023904},
      {"",	0.024133},
      {"",	0.034493},
      {"",	0.031733},
      {"",	0.017932},
      {"",	0.036924},
      {"",	0.086345},
      {"",	0.036020},
      {"",	0.014800},
      {"",	0.015329},
      {"",	0.039660},
      {"",	0.021045},
      {"",	0.029324},
      {"",	0.180739},
      {"",	0.020917},
      {"",	0.021087},
      {"",	0.016450},
      {"",	0.034270},
      {"",	0.023452},
      {"",	0.135500},
      {"",	0.022743},
      {"",	0.175203},
      {"",	0.029011},
      {"",	0.025235},
      {"",	0.020801},
      {"",	0.335331},
      {"",	0.159704},
      {"",	0.043805},
      {"",	0.082474},
      {"",	0.041141},
      {"",	0.070902},
      {"",	0.019910},
      {"²",	0.029031},
      {"",	0.016313},
      {"",	0.018368},
      {"",	0.125698},
      {"",	0.034810},
      {"",	0.267319},
      {"",	0.112807},
      {"",	0.018520},
      {"",	0.030641},
      {"",	0.348555},
      {"",	0.036569},
      {"",	0.056596},
      {"",	0.016607},
      {"",	0.122909},
      {"",	0.054052},
      {"",	0.268859},
      {"",	0.094117},
      {"",	0.024633},
      {"",	0.096599},
      {"",	0.065257},
      {"",	0.021939},
      {"",	0.019991},
      {"",	0.014644},
      {"",	0.030914},
      {"",	0.062679},
      {"",	0.015188},
      {"",	0.021828},
      {"",	0.649007},
      {"",	0.037749},
      {"",	0.072864},
      {"",	0.053675},
      {"",	0.093825},
      {"",	0.034648},
      {"",	0.018468},
      {"",	0.097010},
      {"",	0.031320},
      {"",	0.034071},
      {"",	0.017071},
      {"",	0.331414},
      {"",	0.236619},
      {"",	0.017806},
      {"",	0.033741},
      {"",	0.071342},
      {"",	0.158870},
      {"",	0.075241},
      {"",	0.082499},
      {"",	0.050505},
      {"",	0.037712},
      {"",	0.033770},
      {"",	0.026064},
      {"",	0.122616},
      {"",	0.072859},
      {"",	0.036769},
      {"",	0.023777},
      {"",	0.018066},
      {"",	0.024433},
      {"",	0.105024},
      {"",	1.503535},
      {"",	0.059178},
      {"",	0.115500},
      {"",	0.106853},
      {"",	0.072786},
      {"",	0.028320},
      {"",	0.024624},
      {"",	0.080762},
      {"",	0.124968},
      {"",	0.103272},
      {"",	0.036130},
      {"",	0.084656},
      {"",	0.023113},
      {"",	0.054908},
      {"",	0.095309},
      {"",	0.057877},
      {"",	0.015462},
      {"",	0.259234},
      {"",	0.030173},
      {"",	0.028259},
      {"",	0.025462},
      {"",	0.081804},
      {"",	0.019704},
      {"",	0.037346},
      {"",	0.172006},
      {"",	0.062596},
      {"",	0.042139},
      {"",	0.030141},
      {"",	0.017032},
      {"",	0.033712},
      {"",	0.097897},
      {"",	0.019585},
      {"",	0.036788},
      {"",	0.040640},
      {"",	0.021701},
      {"",	0.015084},
      {"ñ",	0.019035},
      {"",	0.046922},
      {"",	0.033282},
      {"",	0.142554},
      {"",	0.350021},
      {"",	0.020720},
      {"",	0.117753},
      {"",	0.155168},
      {"",	0.027903},
      {"",	0.034138},
      {"",	0.195193},
      {"",	0.091897},
      {"",	0.025892},
      {"",	0.092087},
      {"",	0.098278},
      {"",	0.206358},
      {"",	0.035629},
      {"",	0.037969},
      {"",	0.059885},
      {"",	0.017369},
      {"",	0.041338},
      {"",	0.042914},
      {"",	0.110517},
      {"",	0.024560},
      {"",	0.023162},
      {"",	0.023874},
      {"",	0.020609},
      {"",	0.227495},
      {"",	0.141367},
      {"",	0.015495},
      {"",	0.022215},
      {"",	0.056262},
      {"",	0.151584},
      {"",	0.061118},
      {"",	0.272770},
      {"",	0.085263},
      {"",	0.016105},
      {"",	0.053881},
      {"",	0.021204},
      {"",	0.093396},
      {"",	0.469347},
      {"",	0.020237},
      {"",	0.034130},
      {"",	0.059852},
      {"",	0.043658},
      {"",	0.017357},
      {"",	0.045179},
      {"",	0.023371},
      {"",	0.110726},
      {"",	0.020201},
      {"",	0.024993},
      {"",	0.029663},
      {"",	0.032246},
      {"",	0.015996},
      {"",	0.026776},
      {"",	0.027477},
      {"",	0.039371},
      {"",	0.016772},
      {"",	0.014455},
      {"",	0.024243},
      {"",	0.048220},
      {"",	0.027036},
      {"",	0.019130},
      {"",	0.022422},
      {"",	0.045496},
      {"",	0.054332},
      {"",	0.042048},
      {"",	0.030469},
      {"",	0.042275},
      {"",	0.016165},
      {"",	0.291129},
      {"",	0.094920},
      {"",	0.043175},
      {"",	0.025828},
      {"",	0.016134},
      {"",	0.034991},
      {"",	0.020823},
      {"",	0.035779},
      {"",	0.018547},
      {"",	0.023479},
      {"",	0.045329},
      {"",	0.297640},
      {"",	0.028177},
      {"",	0.085551},
      {"",	0.153848},
      {"",	0.037319},
      {"",	0.129670},
      {"",	0.049393},
      {"",	0.042645},
      {"",	0.030758},
      {"",	0.041439},
      {"",	0.042696},
      {"ù",	0.028028},
      {"",	0.019105},
      {"",	0.052699},
      {"",	0.014792},
      {"",	0.098239}
    };

  static const short lookup[] =
    {
         0,   -1,   -1,   -1,   -1, -408,    3, -399,
        -2,   -1, -413,    6, -396,   -2,   -1,    7,
         8,   -1,    9,   10,   11,   -1,   -1,   12,
        -1,   13,   14,   -1,   15,   16,   17,   18,
        -1,   19,   20,   -1,   21,   -1,   22,   23,
        24,   -1,   25,   26,   -1,   27,   28,   -1,
        29,   30,   31,   32,   -1,   33,   34,   35,
        36,   -1,   37,   38,   39,   40,   -1,   41,
        42,   43,   44,   -1,   45,   46,   47,   -1,
        -1,   48,   49,   50,   51,   -1,   -1,   -1,
        52,   53,   -1,   54,   55,   56,   57,   -1,
        -1,   58,   59,   60,   -1,   -1,   61,   62,
        -1,   -1,   -1,   63,   64,   65,   -1,   66,
        67,   68,   69,   -1,   -1,   70,   71,   72,
        -1,   73,   74,   75,   76,   -1,   77,   -1,
        78,   79,   -1,   -1,   -1,   80,   81,   -1,
        82,   -1,   83,   -1,   -1,   84,   -1,   85,
        86,   -1,   87,   -1,   88,   -1,   -1,   89,
        90,   91,   -1,   -1,   -1,   -1,   92,   -1,
        -1,   -1,   -1,   93,   94,   -1,   95,   -1,
        96,   -1,   -1,   97,   -1,   98,   99,   -1,
       100,   -1,  101,  102,   -1,  103,   -1,  104,
       105,   -1,   -1,   -1,  106,   -1,   -1,  107,
        -1,  108,  109,   -1,  110,   -1,  111,  112,
        -1,  113,  114,   -1,  115,   -1,  116,   -1,
       117,  118,   -1,  119,   -1,  120,  121,   -1,
       122,   -1,  123,  124,   -1,  125,   -1,  126,
       127,   -1,   -1,   -1,  128,  129,   -1,  130,
        -1,  131,  132,   -1,  133,   -1,  134,  135,
        -1,   -1,   -1,  136,  137,   -1,  138,   -1,
       139,  140,   -1,   -1,   -1,  141,  142,   -1,
       143,   -1,  144,   -1,   -1,  145,   -1,  146,
       147,   -1,  148,   -1,  149,  150,   -1,  151,
        -1,  152,  153,   -1,  154,   -1,  155,  156,
        -1,  157,   -1,  158,  159,   -1,  160,   -1,
       161,  162,   -1,  163,   -1,  164,  165,   -1,
       166,   -1,  167,  168,   -1,  169,   -1,  170,
       171,   -1,  172,   -1,  173,  174,   -1,  175,
        -1,  176,  177,   -1,  178,  179, -737,  182,
        -1,  183,   -1,  184,  185,   -1,  186,   -1,
       187,  188,   -1,  189,   -1,  190,  191,   -1,
       192,   -1,  193,  194,   -1,  195,   -1,  196,
      -220,   -2,   -1,   -1,  197,  198,   -1,  199,
        -1,  200,  201,   -1,   -1,   -1,  202,   -1,
        -1,  203,  204,  205,  206,   -1,   -1,   -1,
       207,  208,   -1,  209,   -1,   -1,  210,   -1,
       211,  212,  213,   -1,   -1,  214,   -1,  215,
       216,   -1,  217,   -1,  218,  219,   -1,  220,
        -1,  221,  222,   -1,  223,   -1,  224,  225,
        -1,  226,   -1,  227,  228,   -1,   -1,   -1,
       229,  230,   -1,  231,   -1,  232,  233,   -1,
       234,   -1,  235,  236,   -1,  237,   -1,  238,
       239,   -1,  240,  241,  242,  243,   -1,  244,
        -1,  245,  246,   -1,  247,   -1,  248,  249,
        -1,  250,   -1,  251,  252,   -1,  253,   -1,
       254,  255,   -1,  256,   -1,  257,   -1,   -1,
       258,   -1,  259,  260,   -1,  261,  262,  263,
       264,   -1,  265,   -1,  266,  267,   -1,  268,
        -1,  269,  270,   -1,  271,   -1,  272,  273,
        -1,  274,   -1,  275,   -1,   -1,  276,  277,
       278,  279,   -1,  280,   -1,  281,   -1,   -1,
       282,   -1,  283,  284,   -1,  285,   -1,  286,
       287,   -1,  288,   -1,  289,  290,   -1,  291,
        -1,  292,  293,   -1,  294,   -1,  295,  296,
        -1,  297,   -1,  298,  299,   -1,  300,   -1,
       301,  302,   -1,  303,   -1,  304,  305,   -1,
       306,   -1,  307,  308,   -1,  309,   -1,  310,
        -1,   -1,  311,   -1,  312,  313,   -1,  314,
        -1,  315,  316,   -1,  317,   -1,  318,   -1,
        -1,  319,   -1,  320,  321,   -1,  322,   -1,
       323,  324,   -1,  325,   -1,  326,  327,   -1,
        -1,   -1,  328,  329,   -1, -975,  -70,   -2,
        -1,   -1,  332,   -1,  333,  334,   -1,  335,
        -1,  336,   -1,   -1,  337,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,  338,   -1,   -1,   -1,
       339,   -1,   -1,  340,   -1,  341,  342,   -1,
       343,   -1,  344,  345,   -1,   -1,   -1,  346,
       347,   -1,  348,   -1,   -1,   -1,   -1,  349,
        -1,   -1,  350,   -1,  351,   -1,  352,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
       353,  354,   -1,   -1,   -1,   -1,   -1,   -1,
       355,   -1,  356,  357,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,  358,   -1,
        -1,   -1,   -1,  359,   -1,   -1,  360,   -1,
       361,   -1,   -1,   -1,   -1,  362,  363,   -1,
       364,  365,  366,   -1,   -1,  367,   -1,  368,
        -1,   -1,   -1,   -1,  369,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,  370,   -1,
        -1,  371,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,  372,   -1,   -1,
       373,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,  374,   -1,   -1,   -1,
        -1,   -1,  375,   -1,  376,   -1,   -1,  377,
        -1,   -1,   -1,  378,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
       379,   -1,  380,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,  381,   -1,
        -1,   -1,   -1,  382,   -1,  383,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,  384,   -1,   -1,  385,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,  386,   -1,
        -1,   -1,   -1,  387,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,  388,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,  389,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,  390,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,  391,   -1,  392,   -1,
        -1,   -1,   -1,   -1,  393,   -1,   -1,   -1,
       394,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
        -1,   -1,   -1,   -1,   -1,   -1,  395,   -1,
        -1,  396,   -1,  397,   -1,   -1,   -1,   -1,
        -1,   -1,  398,   -1,   -1,   -1,   -1,   -1,
        -1,  399
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      register int key = b5Hash (str, len);

      if (key <= MAX_HASH_VALUE && key >= 0)
        {
          register int index = lookup[key];

          if (index >= 0)
            {
              register const char *s = wordlist[index].name;

              if (*str == *s && !strncmp (str + 1, s + 1, len - 1))
                return &wordlist[index];
            }
          else if (index < -TOTAL_KEYWORDS)
            {
              register int offset = - 1 - TOTAL_KEYWORDS - index;
              register const struct charHz *wordptr = &wordlist[TOTAL_KEYWORDS + lookup[offset]];
              register const struct charHz *wordendptr = wordptr + -lookup[offset + 1];

              while (wordptr < wordendptr)
                {
                  register const char *s = wordptr->name;

                  if (*str == *s && !strncmp (str + 1, s + 1, len - 1))
                    return wordptr;
                  wordptr++;
                }
            }
        }
    }
  return 0;
}

static int j_code3(const char * buff, int count)
{
	const unsigned char *phz;
	double c_gb, c_b5;		/* Frequency count for char */
//	double o_gb, o_b5;		/* Holes in opposit encode.*/
					/* If we test user defined chars. */
	const struct charHz * temphz;	/* temperary char frequency. */

	c_gb = c_b5 = 0;

	for(phz=(unsigned char * /*FIXIT*/)buff; (int)phz < ((int)buff + count); phz++){
		if(*phz & 0x80){
			/* First Byte not in GB range. 
			 * We don't do user defined chars. 
			 * Should not simply return if we consider
			 * user defined chars
			 * */
			if((*phz >= 0xF8 && *phz <= 0xF9) && ( (*(phz+1) >= 0x40 && *(phz+1) <= 0x7E) || (*(phz+1) >= 0xA1 && *(phz+1) <= 0xFE ))){
				return BIG5_CODE;
			}
			/* Second Byte not in GB range. */
			if ((*phz >= 0xA1 && *phz <= 0xF7) && (* (phz+1) >= 0x40 && *(phz+1) <= 0x7E)){

				return BIG5_CODE;
			}
			/* GB holes in first byte: 0xA8-0xAF. */
			if((*phz >= 0xA8 && *phz <= 0xAF) && ((*(phz+1) >= 0x40 && *(phz+1) <= 0x7E ) || (*(phz+1) >= 0xA1 && *(phz+1) <= 0xFE ))){

				return BIG5_CODE;
			}

			/* Valid GB and BIG5? see which one is more likely. */
			if ((*phz >= 0xA1 && *phz <= 0xF7) && (* (phz+1) >= 0xA1 && *(phz+1) <= 0xFE)){

				temphz = inGB((char * /*FIXIT*/)phz, 2);
				if(temphz)
					c_gb += temphz->percent;

				temphz = inBig5((char * /*FIXIT*/)phz, 2);
				if(temphz)
					c_b5 += temphz->percent;

				/* Every valid GB and BIG5 codes
				 * are tested. we jump 2 here.
				 */
				phz++;
			}
		}
		/* Anything else are either ASCII or screwed
		 * text. We pass that one char. 
		 */
	} /* for loop end */
	
#ifdef DEBUG
printf("GB = %f, \t BIG5 = %f\n", c_gb, c_b5);
#endif

	/* Assume that == is GB code. Prove me wrong! :) */
	if( c_gb >= c_b5 )
		return GB_CODE;
	else
		return BIG5_CODE;
}

/*
 *This function replaced by j_code3
 *2000.8.31
 *Yu Guanghui <ygh@dlut.edu.cn>
 */
/*
int j_code2(char * buff,int count)
{
	unsigned char * phz;
	int c_gb=0;
	
	for(phz=buff;phz<(buff+count);phz++){
		if(*phz & 0x80){
			if ((*phz >= 0xF8 && *phz <= 0xF9) && ( (*(phz+1) >= 0x40 && *(phz+1) <= 0x7E) || (*(phz+1) >= 0xA1 && *(phz+1) <= 0xFE ))){
				return BIG5_CODE;
			}
			if ((*phz >= 0xA1 && *phz <= 0xF7) && (* (phz+1) >= 0x40 && *(phz+1) <= 0x7E)){
				return BIG5_CODE;
			}
			if ((*phz >= 0xA1 && *phz <= 0xF7) && (* (phz+1) >= 0xA1 && *(phz+1) <= 0xFE)){
				c_gb++;
			}
			phz++;
			continue;
		}
	}

	if( c_gb ==0 ){
		return OTHER_CODE;
	}else {
		return GB_CODE;
	}

}
*/
